(*
    Copyright (C) 2012 Christophe Calvès

    This file is part of WC.

    WC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*)


(************************
 *                      *
 *     FONCTIONAL       *
 *                      *
 ************************)

let swap f x y = f y x
let curry f x y = f (x,y)
let uncurry f (x,y) = f x y
let id  x = x
let cst x _ = x
let (|>) f g x = g (f x)
let (<|) f g x = f (g x)


let fst (x,_) = x
let snd (_,x) = x

(************************
 *                      *
 *       MAP/SET        *
 *                      *
 ************************)


module OrdString =
struct
 type t = string
 let compare = compare
end

module SetString = Set.Make ( OrdString )
module MapString = Map.Make ( OrdString )


module OrdInt =
struct
 type t = int
 let compare = compare
end

module SetInt = Set.Make ( OrdInt )
module MapInt = Map.Make ( OrdInt )

(* For 3.11 *)
let mapIntBindings    m = MapInt.fold    (fun k v l -> (k,v) :: l) m []
let mapStringBindings m = MapString.fold (fun k v l -> (k,v) :: l) m []


(************************
 *                      *
 *       BASE2          *
 *                      *
 ************************)


let strBaseTwo x =
 let rec aux = function
  | 0 -> "O"
  | 1 -> "T"
  | n ->  (aux (n/2)) ^ (if n mod 2 = 0 then "O" else "T")
 in let s = aux x
    in (String.make (max 0 (10 - (String.length s))) 'O') ^ s



(************************
 *                      *
 *       LIST           *
 *                      *
 ************************)


(************************
 *                      *
 *       OPTION         *
 *                      *
 ************************)


let omap f = function
 | None   -> None
 | Some x -> Some (f x)


(************************
 *                      *
 *       FILES          *
 *                      *
 ************************)


let stringOfFile s =
    let cin = try (open_in s) with _ -> failwith ("File " ^ s ^ " not found!")
 in let rec aux e = try (aux (e ^ (input_line cin) ^ "\n"))
                    with End_of_file -> (close_in cin ; e)
 in aux ""
