%{

(*
    Copyright (C) 2012 Christophe Calvès

    This file is part of WC.

    WC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*)


open Ast ;;
open Base ;;

%}

%token LParL
%token LParR
%token LSemiCol

%token <string> LIdent
%token <string> LString
%token <int>    LInt
%token <bool>   LBool
%token LCst
%token LFun
%token LArr
%token LExt
%token LDbCol
%token LRec

%token LIf
%token LThen
%token LElse

%token LLet
%token LEq
%token LIn
%token LAnd

%token LEOF

%start ast
%type <(string , Ast.ibs) Ast.ast> ast



%%
ast : externs app LEOF { AAst ((List.rev $1) , $2) }

externs :                          { [] }
 | externs LExt LIdent LInt LDbCol { ($3,$4) :: $1 }

app:
 | simple                  { $1 }
 | app simple              { AApp ( $1 , [$2] ) }
 | app LSemiCol simple     { ASeq [$1 ; $3]     }
;


simple : 
   LBool                   { ADat (Bool $1)    }
 | LString                 { ADat (String $1 ) }
 | LInt                    { ADat (Int    $1 ) }
 | LIdent                  { AVar $1 }


 | LFun idents LArr simple             { AFun ((List.rev $2) , $4)  }
 | LParL app  LParR                    { $2              }
 | LLet  lets LIn simple               { ALetIn ( List.rev $2 , $4) }
 | LLet LRec LIdent LEq app LIn simple { ALetRecIn ($3,$5,$7)       }

 | LIf app LThen app LElse simple      { AIf ($2,$4,$6)             }
 ;

idents :
   LIdent                 { [ $1 ]   }
 | idents LIdent          { $2 :: $1 }
;

letunique :
   LIdent LEq app         { ( $1 , $3 ) }
;

lets :
   letunique              { [ $1 ]   }
 | lets LAnd letunique    { $3 :: $1 } 
;

%% 



