(*
    Copyright (C) 2012 Christophe Calvès

    This file is part of WC.

    WC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*)


open Base
open Ast
open Comp
open Statem


let ofIBS = function
 | Int    i     -> (string_of_int i)
 | String s     -> s
 | Bool   true  -> "1"
 | Bool   false -> "0"


let ofSuper f aritytable x =
 let rec auxAtm = function
   | Cst s -> (match (try (MapString.find s aritytable)
                      with Not_found -> failwith (s ^ " was not found")
                     ) with
                | 0     -> (false , s)
                | arity -> (true , "mkclosure " ^ (string_of_int arity) ^ " " ^ s ^ "\n" )
              )
   | Var n -> (false, "$"^(string_of_int n))
   | Dat d -> (false , f d                 )

 and aux = function
   | SAtm a       -> let (b,v) = auxAtm a
                     in if b (* use acc? *)
                        then v
                        else ("ACC=" ^ v ^ "\n")
   | SApp (a, sl) -> let s = aux (SAtm a)
                     in List.fold_left auxApply s sl

 and auxApply s = function
   | SAtm a -> let (b,v) = auxAtm a
               in if b (* use acc? *)
                  then complexApply s v
                  else (s ^ "apply \"$ACC\" \"" ^ v ^ "\"\n")
   | x      -> let v = aux x
               in complexApply s v
 and complexApply s v = (  s
                         ^ "stackpush \"$ACC\"\n"
                         ^ v
                         ^ "TMP=\"$ACC\"\n"
                         ^ "stackpop\n"
                         ^ "apply \"$ACC\" \"$TMP\"\n"
                        )
 in aux x
                   

let ofProc f aritytable = function
 | Proc (s,n, Some c) -> (  "##---------  " ^ s ^ "  ----------\n" ^ s ^ " () {\n"
                           ^ (ofSuper f aritytable c) ^ "}\n\n"
                         )
 | _                  -> ""




let ofProg f (Prog (l,a)) =
    let aritytable = List.fold_left (fun m (Proc (s,n,_)) -> MapString.add s n m)
                                    MapString.empty
                                    l
 in let lsc = List.map (ofProc f aritytable) l
 in let scs = List.fold_left (^) "" lsc
 in (  scs
     ^ "##---------  TERM  ---------\n"
     ^ (ofSuper f aritytable a)
    )


let standAloneOfProg f libpath x =
   "#!/bin/sh\n"
 ^ (stringOfFile (libpath ^ "lib.sh"))
 ^ (stringOfFile (libpath ^ "extern.sh"))
 ^ "
##########################
#                        #
#        PROGRAM         #
#                        #
##########################
"
 ^ (ofProg f x)
