(*
    Copyright (C) 2012 Christophe Calvès

    This file is part of WC.

    WC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*)


open Base
open Ast
open Comp
open Statem


let ofIBS = function
 | Int    i     -> (string_of_int i)
 | String s     -> "{" ^ s                 ^ "}"
 | Bool   true  -> "1"
 | Bool   false -> "0"


let saveAcc () = sget >>= (fun i ->
                 sput (i + 1) >>
                 sreturn ("\\val" ^ (strBaseTwo i)) )

let ofSuper f aritytable x =
 let rec auxAtm = function
   | Cst s -> (match (try (MapString.find s aritytable)
                      with Not_found -> failwith (s ^ " was not found")
                     ) with
                | 0     -> sreturn (false , "\\" ^ s)
                | arity -> sreturn (true , "\\mkclosure{" ^ (string_of_int arity) ^ "}{\\" ^ s ^ "}%\n" )
              )
   | Var n -> sreturn (false, "#"^(string_of_int n))
   | Dat d -> sreturn (false , f d                 )

 and aux = function
   | SAtm a       -> auxAtm a >>= (fun (b,v) ->
                     if b (* use acc? *)
                     then sreturn v
                     else sreturn ("\\gdef\\acc{" ^ v ^ "}%\n" ) )
   | SApp (a, sl) -> aux (SAtm a) >>= (fun s ->
                     sfoldl auxApply s sl )

 and auxApply s = function
   | SAtm a -> auxAtm a    >>= (fun (b,v) ->
               if b (* use acc? *)
               then saveAcc () >>= complexApply s v
               else sreturn (s ^ "\\expandafter\\apply\\expandafter{\\acc}{" ^ v ^ "}%\n")
               )
   | x      -> saveAcc ()      >>= (fun nv ->
               slocal (aux x)  >>= (fun v  ->
               complexApply s v nv))
 and complexApply s v nv = sreturn (
                           (  s
                            ^ "\\expandafter\\stackpush\\expandafter{\\acc}%\n"
                            ^ v
                            ^ "\\let\\tmp=\\acc%\n"
                            ^ "\\stackpop%\n"
                            ^ "\\expandafter\\swapa\\expandafter{\\tmp}"
                            ^ "{\\expandafter\\apply\\expandafter{\\acc}}%\n"
                           ) )
 in srun (aux x) 1

                   

let ofProc f aritytable = function
 | Proc (s,n, Some c) -> let rec argpat = function
                          | x when x > n -> ""
                          | x            -> "#" ^ (string_of_int x) ^ (argpat (x + 1))
                         in   (  "%%---------  " ^ s ^ "  ----------\n\\def\\" ^ s ^ (argpat 1)
                               ^ "%\n{%\n"
                               ^ (ofSuper f aritytable c) ^ "}\n%\n"
                              )
 | _                  -> ""




let ofProg f (Prog (l,a)) =
    let aritytable = List.fold_left (fun m (Proc (s,n,_)) -> MapString.add s n m)
                                    MapString.empty
                                    l
 in let lsc = List.map (ofProc f aritytable) l
 in let scs = List.fold_left (^) "" lsc
 in (  scs
     ^ "%%---------  TERM  ---------\n"
     ^ (ofSuper f aritytable a)
    )


let standAloneOfProg f libpath x =
   "\\tracingcommands=1%\n"
 ^ "\\tracingmacros=1%\n"
 ^ (stringOfFile (libpath ^ "lib.tex"))
 ^ (stringOfFile (libpath ^ "extern.tex"))
 ^ "%
% %%%%%%%%%%%%%%%%%%%%%
% %      PROGRAM      %
% %%%%%%%%%%%%%%%%%%%%%
%
%
"
 ^ (ofProg f x)
 ^ "\\end\n"
